@@ -1,13 +1,22 @@
-#include <locale.h>
-#ifndef _WIN32
+
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+#ifdef HAVE_LOCALE_H
+# include <locale.h>
+#endif
+
+#ifdef HAVE_LANGINFO_H
 # include <langinfo.h>
-#else
-# define WIN32_LEAN_AND_MEAN
-# include <windows.h>
-# undef WIN32_LEAN_AND_MEAN
-#endif /* _WIN32 */
-#include "ecore_private.h"
+#endif
+
 #include "Ecore.h"
+#include "ecore_private.h"
+
+#ifdef HAVE_EVIL
+# include <Evil.h>
+#endif
 
 static const char *_ecore_magic_string_get(Ecore_Magic m);
 static int _ecore_init_count = 0;
@@ -49,7 +58,9 @@ ecore_init(void)
 {
    if (++_ecore_init_count == 1)
      {
+#ifdef HAVE_LOCALE_H
 	setlocale(LC_CTYPE, "");
+#endif
 	/*
 	if (strcmp(nl_langinfo(CODESET), "UTF-8"))
 	  {
@@ -58,10 +69,8 @@ ecore_init(void)
 	 */
 	if (getenv("ECORE_FPS_DEBUG")) _ecore_fps_debug = 1;
 	if (_ecore_fps_debug) _ecore_fps_debug_init();
-#ifndef _WIN32
 	_ecore_signal_init();
-        _ecore_exe_init();
-#endif
+	_ecore_exe_init();
      }
 
    return _ecore_init_count;
@@ -86,18 +95,14 @@ ecore_shutdown(void)
    if (_ecore_fps_debug) _ecore_fps_debug_shutdown();
    _ecore_poller_shutdown();
    _ecore_animator_shutdown();
-#ifndef _WIN32
    _ecore_exe_shutdown();
-#endif
    _ecore_idle_enterer_shutdown();
    _ecore_idle_exiter_shutdown();
    _ecore_idler_shutdown();
    _ecore_timer_shutdown();
    _ecore_event_shutdown();
    _ecore_main_shutdown();
-#ifndef _WIN32
    _ecore_signal_shutdown();
-#endif
 
    return _ecore_init_count;
 }
@@ -170,28 +175,26 @@ _ecore_magic_string_get(Ecore_Magic m)
 /* account for other apps and multitasking... */
 
 static int _ecore_fps_debug_init_count = 0;
-#ifndef _WIN32
 static int _ecore_fps_debug_fd = -1;
 unsigned int *_ecore_fps_runtime_mmap = NULL;
-#else
-static HANDLE _ecore_fps_debug_fd = NULL;
-static HANDLE _ecore_fps_debug_fm = NULL;
-unsigned int *_ecore_fps_runtime_mmap = NULL;
-#endif /* _WIN32 */
 
 void
 _ecore_fps_debug_init(void)
 {
-   char buf[4096];
-#ifdef _WIN32
+   char  buf[4096];
    char *tmp;
-#endif /* _WIN32 */
+   int   pid;
 
    _ecore_fps_debug_init_count++;
    if (_ecore_fps_debug_init_count > 1) return;
 
-#ifndef _WIN32
-   snprintf(buf, sizeof(buf), "/tmp/.ecore_fps_debug-%i", (int)getpid());
+#ifndef HAVE_EVIL
+   tmp = "/tmp";
+#else
+   tmp = (char *)evil_tmpdir_get ();
+#endif /* HAVE_EVIL */
+   pid = (int)getpid();
+   snprintf(buf, sizeof(buf), "%s/.ecore_fps_debug-%i", tmp, pid);
    _ecore_fps_debug_fd = open(buf, O_CREAT | O_TRUNC | O_RDWR, 0644);
    if (_ecore_fps_debug_fd < 0)
      {
@@ -207,46 +210,9 @@ _ecore_fps_debug_init(void)
 				       PROT_READ | PROT_WRITE,
 				       MAP_SHARED,
 				       _ecore_fps_debug_fd, 0);
+	if (_ecore_fps_runtime_mmap == MAP_FAILED)
+	  _ecore_fps_runtime_mmap = NULL;
      }
-#else
-   tmp = getenv("TMP");
-   if (!tmp) tmp = getenv("TEMP");
-   if (!tmp) tmp = getenv("USERPROFILE");
-   if (!tmp) tmp = getenv("windir");
-   if (!tmp) tmp = "C:";
-   snprintf(buf, sizeof(buf), "%s/.ecore_fps_debug-%i", tmp, (int)GetCurrentProcessId());
-   _ecore_fps_debug_fd = CreateFile(buf,
-                                    FILE_READ_DATA | FILE_WRITE_DATA,
-                                    FILE_SHARE_READ | FILE_SHARE_WRITE,
-                                    NULL,
-                                    CREATE_NEW,// | TRUNCATE_EXISTING,
-                                    FILE_ATTRIBUTE_NORMAL,
-                                    NULL);
-   if (_ecore_fps_debug_fd)
-     {
-	unsigned int zero = 0;
-        DWORD        out;
-
-        _ecore_fps_debug_fm = CreateFileMapping(_ecore_fps_debug_fd,
-                                                NULL,
-                                                PAGE_READWRITE,
-                                                0,
-                                                (DWORD)sizeof(unsigned int),
-                                                NULL);
-        if (_ecore_fps_debug_fm)
-          {
-             WriteFile(_ecore_fps_debug_fd,
-                       &zero, sizeof(unsigned int),
-                       &out, NULL);
-             _ecore_fps_runtime_mmap = MapViewOfFile(_ecore_fps_debug_fm,
-                                                     FILE_MAP_WRITE,
-                                                     0, 0,
-                                                     sizeof(unsigned int));
-          }
-        else
-          CloseHandle(_ecore_fps_debug_fd);
-     }
-#endif /* _WIN32 */
 }
 
 void
@@ -257,27 +223,16 @@ _ecore_fps_debug_shutdown(void)
    if (_ecore_fps_debug_fd >= 0)
      {
 	char buf[4096];
-#ifdef _WIN32
         char *tmp;
+        int   pid;
 
-        tmp = getenv("TMP");
-        if (!tmp) tmp = getenv("TEMP");
-        if (!tmp) tmp = getenv("USERPROFILE");
-        if (!tmp) tmp = getenv("windir");
-        if (!tmp) tmp = "C:/";
-	snprintf(buf, sizeof(buf), "%s/.ecore_fps_debug-%i", tmp, (int)GetCurrentProcessId());
-	if (_ecore_fps_runtime_mmap)
-	  {
-	     UnmapViewOfFile(_ecore_fps_runtime_mmap);
-	     _ecore_fps_runtime_mmap = NULL;
-	  }
-        CloseHandle(_ecore_fps_debug_fm);
-        CloseHandle(_ecore_fps_debug_fd);
-	_ecore_fps_debug_fd = NULL;
-	_ecore_fps_debug_fm = NULL;
-	_unlink(buf);
+#ifndef HAVE_EVIL
+   tmp = "/tmp";
 #else
-	snprintf(buf, sizeof(buf), "/tmp/.ecore_fps_debug-%i", (int)getpid());
+   tmp = (char *)evil_tmpdir_get ();
+#endif /* HAVE_EVIL */
+   pid = (int)getpid();
+	snprintf(buf, sizeof(buf), "%s/.ecore_fps_debug-%i", tmp, pid);
 	unlink(buf);
 	if (_ecore_fps_runtime_mmap)
 	  {
@@ -286,7 +241,6 @@ _ecore_fps_debug_shutdown(void)
 	  }
 	close(_ecore_fps_debug_fd);
 	_ecore_fps_debug_fd = -1;
-#endif /* _WIN32 */
      }
 }
 
