@@ -2,9 +2,13 @@
  * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
  */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
 #ifdef _WIN32
 # include <winsock2.h>
-#endif /* _WIN32 */
+#endif
 
 #include <math.h>
 #include <sys/time.h>
@@ -110,16 +114,17 @@ ecore_main_loop_quit(void)
  *
  * Normally the return value from the @p func is "zero means this handler is
  * finished and can be deleted" as is usual for handler callbacks.  However,
- * if the @p buf_func is supplied, then the return value from the @p func is "non
- * zero means the handler should be called again in a tight loop".
+ * if the @p buf_func is supplied, then the return value from the @p func is 
+ * "non zero means the handler should be called again in a tight loop".
  *
  * @p buf_func is called during event loop handling to check if data that has
  * been read from the file descriptor is in a buffer and is available to
  * read.  Some systems (notably xlib) handle their own buffering, and would
  * otherwise not work with select().  These systems should use a @p buf_func.
  * This is a most annoying hack, only ecore_x uses it, so refer to that for
- * an example.  NOTE - @p func should probably return "one" always if @p buf_func
- * is used, to avoid confusion with the other return walue semantics.
+ * an example.  NOTE - @p func should probably return "one" always if 
+ * @p buf_func is used, to avoid confusion with the other return value 
+ * semantics.
  *
  * @param   fd       The file descriptor to watch.
  * @param   flags    To watch it for read (@c ECORE_FD_READ) and/or
@@ -346,9 +351,7 @@ _ecore_main_select(double timeout)
 	     if (fdh->fd > max_fd) max_fd = fdh->fd;
 	  }
      }
-#ifndef _WIN32
    if (_ecore_signal_count_get()) return -1;
-#endif /* _WIN32 */
    ret = select(max_fd + 1, &rfds, &wfds, &exfds, t);
    if (ret < 0)
      {
@@ -532,10 +535,8 @@ _ecore_main_loop_iterate_internal(int once_only)
      }
    /* any timers re-added as a result of these are allowed to go */
    _ecore_timer_enable_new();
-#ifndef _WIN32
    /* process signals into events .... */
    while (_ecore_signal_count_get()) _ecore_signal_call();
-#endif /* _WIN32 */
    if (_ecore_event_exist())
      {
 	have_event = 1;
@@ -555,9 +556,7 @@ _ecore_main_loop_iterate_internal(int once_only)
 	if (_ecore_main_select(0.0) > 0) have_event = 1;
 
 	if (_ecore_main_win32_message(0.0) > 0) have_msg = 1;
-#ifndef _WIN32
 	if (_ecore_signal_count_get() > 0) have_signal = 1;
-#endif /* _WIN32 */
 
 	if (have_signal || have_event || have_msg)
 	  goto process_events;
@@ -566,6 +565,7 @@ _ecore_main_loop_iterate_internal(int once_only)
    /* if these calls caused any buffered events to appear - deal with them */
    while (_ecore_main_fd_handlers_buf_call());
    while (_ecore_main_win32_message_buf_call());
+
    /* if ther are any - jump to processing them */
    if (_ecore_event_exist())
      {
@@ -608,9 +608,7 @@ _ecore_main_loop_iterate_internal(int once_only)
 	       {
 		  if (_ecore_main_select(-1.0) > 0) have_event = 1;
 		  if (_ecore_main_win32_message(-1.0) > 0) have_msg = 1;
-#ifndef _WIN32
 		  if (_ecore_signal_count_get() > 0) have_signal = 1;
-#endif /* _WIN32 */
 	       }
 	     /* idlers */
 	     else
@@ -621,9 +619,7 @@ _ecore_main_loop_iterate_internal(int once_only)
 		       if (_ecore_event_exist()) break;
 		       if (_ecore_main_select(0.0) > 0) have_event = 1;
 		       if (_ecore_main_win32_message(0.0) > 0) have_msg = 1;
-#ifndef _WIN32
 		       if (_ecore_signal_count_get() > 0) have_signal = 1;
-#endif /* _WIN32 */
 		       if (have_event || have_signal || have_msg) break;
 		       next_time = _ecore_timer_next_get();
 		       if (next_time >= 0) goto start_loop;
@@ -639,9 +635,7 @@ _ecore_main_loop_iterate_internal(int once_only)
 	       {
 		  if (_ecore_main_select(next_time) > 0) have_event = 1;
 		  if (_ecore_main_win32_message(next_time) > 0) have_msg = 1;
-#ifndef _WIN32
 		  if (_ecore_signal_count_get() > 0) have_signal = 1;
-#endif /* _WIN32 */
 	       }
 	     /* idlers */
 	     else
@@ -654,9 +648,7 @@ _ecore_main_loop_iterate_internal(int once_only)
 		       if (_ecore_event_exist()) break;
 		       if (_ecore_main_select(0.0) > 0) have_event = 1;
 		       if (_ecore_main_win32_message(0.0) > 0) have_msg = 1;
-#ifndef _WIN32
 		       if (_ecore_signal_count_get() > 0) have_signal = 1;
-#endif /* _WIN32 */
 		       if (have_event || have_signal || have_msg) break;
 		       cur_time = ecore_time_get();
 		       t = ecore_time_get() - cur_time;
@@ -682,11 +674,8 @@ _ecore_main_loop_iterate_internal(int once_only)
    if (have_event) _ecore_main_fd_handlers_call();
    do
      {
-#ifndef _WIN32
 	/* process signals into events .... */
 	while (_ecore_signal_count_get()) _ecore_signal_call();
-#endif /* _WIN32 */
-
 	/* handle events ... */
 	_ecore_event_call();
 	_ecore_main_fd_handlers_cleanup();
@@ -694,6 +683,11 @@ _ecore_main_loop_iterate_internal(int once_only)
    while (_ecore_main_fd_handlers_buf_call());
    if (have_msg) _ecore_main_win32_message_buf_call();
    while (_ecore_main_win32_message_buf_call());
-   if (once_only) _ecore_idle_enterer_call();
+/* ok - too much optimising. let's call idle enterers more often. if we
+ * have events that place more events or jobs etc. on the event queue
+ * we may never get to call an idle enterer
+   if (once_only)
+ */
+     _ecore_idle_enterer_call();
    in_main_loop--;
 }
