@@ -2,15 +2,16 @@
  * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
  */
 
-#ifdef _WIN32
-# define WIN32_LEAN_AND_MEAN
-# include <windows.h>
-# undef WIN32_LEAN_AND_MEAN
-#else
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+#ifdef HAVE_SYS_TIME_H
 # include <sys/time.h>
 #endif
-#include "ecore_private.h"
+
 #include "Ecore.h"
+#include "ecore_private.h"
 
 /* FIXME: clock_gettime() is an option... */
 
@@ -22,22 +23,12 @@
 EAPI double
 ecore_time_get(void)
 {
-#ifdef _WIN32
-   FILETIME ft;
-   double     time;
-
-   GetSystemTimeAsFileTime(&ft);
-   time = (double)ft.dwLowDateTime + 4294967296.0 * (double)ft.dwHighDateTime;
-
-   return time / 10000000;
-#else
-# ifdef HAVE_GETTIMEOFDAY
+#ifdef HAVE_GETTIMEOFDAY
    struct timeval      timev;
 
    gettimeofday(&timev, NULL);
    return (double)timev.tv_sec + (((double)timev.tv_usec) / 1000000);
-# else
-#  error "Your platform isn't supported yet"
-# endif
-#endif /* _WIN32 */
+#else
+# error "Your platform isn't supported yet"
+#endif
 }
