@@ -38,22 +38,6 @@ _ecore_evas_directfb_render(Ecore_Evas *ee)
    updates = evas_render_updates(ee->evas);
    if (updates)
      {
-	DFBRegion region;
-	IDirectFBSurface *surface;
-	Evas_List *l;
-	
-	surface = ee->engine.directfb.window->surface;
-	for (l = updates; l; l = l->next)
-	  {
-	     Evas_Rectangle *rect;
-	     
-	     rect = l->data;
-	     region.x1 = rect->x;
-	     region.y1 = rect->y;
-	     region.x2 = rect->x + rect->w - 1;
-	     region.y2 = rect->y + rect->h - 1;
-	     surface->Flip(surface, &region,DSFLIP_BLIT);
-	  }
 	evas_render_updates_free(updates);
 	_ecore_evas_idle_timeout_update(ee);
      }
@@ -352,9 +336,8 @@ _ecore_evas_directfb_init(void)
 static void
 _ecore_evas_directfb_free(Ecore_Evas *ee)
 {
-   ecore_directfb_window_del(ee->engine.directfb.window);
-   free(ee->engine.directfb.window);
    ecore_evases_hash = evas_hash_del(ecore_evases_hash, _ecore_evas_directfb_winid_str_get(ee->engine.directfb.window->id), ee);
+   ecore_directfb_window_del(ee->engine.directfb.window);
    ecore_evases = _ecore_list2_remove(ecore_evases, ee);   
    _ecore_evas_directfb_shutdown();
    ecore_directfb_shutdown();
@@ -536,6 +519,7 @@ static const Ecore_Evas_Engine_Func _ecore_directfb_engine_func =
      _ecore_evas_directfb_hide,	        /* hide */
      NULL,				/* raise */
      NULL,				/* lower */
+     NULL,				/* activate */
      NULL,				/* title set */
      NULL,				/* name class set */
      NULL,				/* size min */
@@ -616,7 +600,6 @@ ecore_evas_directfb_new(const char *disp_name, int windowed, int x, int y, int w
      {
 	einfo->info.dfb = ecore_directfb_interface_get();
 	einfo->info.surface = window->surface;
-	einfo->info.flags = DSDRAW_BLEND;
 	evas_engine_info_set(ee->evas, (Evas_Engine_Info *)einfo);
      }
    ecore_evases = _ecore_list2_prepend(ecore_evases, ee);
