@@ -89,7 +89,11 @@ ecore_imf_context_default_id_by_canvas_type_get(const char *canvas_type)
    int best_goodness = 0;
 
    id = getenv("ECORE_IMF_MODULE");
-   if (id && ecore_imf_module_get(id)) return id;
+   if (id)
+     {
+	if (strcmp(id, "none") == 0) return NULL;
+	if (ecore_imf_module_get(id)) return id;
+     }
 
    modules = ecore_imf_module_available_get();
    if (!modules) return NULL;
@@ -551,7 +555,7 @@ ecore_imf_context_new(const Ecore_IMF_Context_Class *ctxc)
 {
    Ecore_IMF_Context *ctx;
 
-   if (!ctx) return NULL;
+   if (!ctxc) return NULL;
    ctx = malloc(sizeof(Ecore_IMF_Context));
    if (!ctx) return NULL;
    ECORE_MAGIC_SET(ctx, ECORE_MAGIC_CONTEXT);
