@@ -2,13 +2,24 @@
  * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
  */
 
-#include <stdio.h>
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
 
-#include <Ecore.h>
+#include <stdlib.h>
+#include <stdio.h>   /* for printf */
 
+#define WIN32_LEAN_AND_MEAN
+#include <windows.h>
+#undef WIN32_LEAN_AND_MEAN
+
+#include "Ecore_Win32.h"
 #include "ecore_win32_private.h"
 
 
+/***** Private declarations *****/
+
+
 typedef enum _Ecore_Win32_Window_Z_Order Ecore_Win32_Window_Z_Order;
 enum _Ecore_Win32_Window_Z_Order
 {
@@ -18,122 +29,17 @@ enum _Ecore_Win32_Window_Z_Order
   ECORE_WIN32_WINDOW_Z_ORDER_TOPMOST
 };
 
+static Ecore_Win32_Window *ecore_win32_window_internal_new(Ecore_Win32_Window *parent,
+                                                           int                 x,
+                                                           int                 y,
+                                                           int                 width,
+                                                           int                 height,
+                                                           DWORD               style);
 
-static Ecore_Win32_Window *
-ecore_win32_window_internal_new(Ecore_Win32_Window *parent,
-                                int                 x,
-                                int                 y,
-                                int                 width,
-                                int                 height,
-                                DWORD               style)
-{
-   RECT                        rect;
-   struct _Ecore_Win32_Window *w;
-   int                         minimal_width;
-   int                         minimal_height;
 
-   w = (struct _Ecore_Win32_Window *)calloc(1, sizeof(struct _Ecore_Win32_Window));
-   if (!w)
-     return NULL;
+/***** API *****/
 
-   printf (" *** ecore_win32_window_new : %p  %d %d %d\n",
-           w,
-           width, height, GetSystemMetrics(SM_CXMIN));
-   rect.left = 0;
-   rect.top = 0;
-   rect.right = width;
-   rect.bottom = height;
-   if (!AdjustWindowRect(&rect, style, FALSE))
-     {
-        free(w);
-        return NULL;
-     }
-   printf (" * ecore : new debut : %ld %d %d\n",
-           rect.right - rect.left, GetSystemMetrics(SM_CXMIN), GetSystemMetrics(SM_CYMIN));
-
-   minimal_width = GetSystemMetrics(SM_CXMIN);
-   minimal_height = GetSystemMetrics(SM_CYMIN);
-/*    if (((rect.right - rect.left) < minimal_width) || */
-/*        ((rect.bottom - rect.top) < minimal_height)) */
-/*      { */
-/*         fprintf (stderr, "[Ecore] [Win32] ERROR !!\n"); */
-/*         fprintf (stderr, "                Wrong size %ld\n", rect.right - rect.left); */
-/*         free(w); */
-/*         return NULL; */
-/*      } */
-   if ((rect.right - rect.left) < minimal_width)
-     {
-       rect.right = rect.left + minimal_width;
-     }
-
-   w->window = CreateWindow(ECORE_WIN32_WINDOW_CLASS, "",
-                            style,
-                            x, y,
-                            rect.right - rect.left,
-                            rect.bottom - rect.top,
-                            parent ? ((struct _Ecore_Win32_Window *)parent)->window : NULL,
-                            NULL, _ecore_win32_instance, NULL);
-   if (!w->window)
-     {
-        free(w);
-        return NULL;
-     }
-
-   if (ecore_list_append(_ecore_win32_windows_list, w) == FALSE)
-     {
-        ecore_win32_ddraw_shutdown(w);
-        DestroyWindow(w->window);
-        free(w);
-        return NULL;
-     }
-
-   w->backend = ECORE_WIN32_BACKEND_NONE;
-
-   w->min_width   = 0;
-   w->min_height  = 0;
-   w->max_width   = 32767;
-   w->max_height  = 32767;
-   w->base_width  = -1;
-   w->base_height = -1;
-   w->step_width  = -1;
-   w->step_height = -1;
-
-   w->state.iconified         = 0;
-   w->state.modal             = 0;
-   w->state.sticky            = 0;
-   w->state.maximized_vert    = 0;
-   w->state.maximized_horz    = 0;
-   w->state.shaded            = 0;
-   w->state.hidden            = 0;
-   w->state.fullscreen        = 0;
-   w->state.above             = 0;
-   w->state.below             = 0;
-   w->state.demands_attention = 0;
-
-   w->type.desktop = 0;
-   w->type.dock    = 0;
-   w->type.toolbar = 0;
-   w->type.menu    = 0;
-   w->type.utility = 0;
-   w->type.splash  = 0;
-   w->type.dialog  = 0;
-   w->type.normal  = 0;
-
-   w->pointer_is_in = 0;
-   w->borderless    = 0;
-   w->iconified     = 0;
-   w->fullscreen    = 0;
-
-   printf (" *** ecore_win32_window_new fin : (%d %d) (%d %d)\n",
-           w->min_width,
-           w->min_height,
-           w->max_width,
-           w->max_height);
-
-   return w;
-}
-
-EAPI Ecore_Win32_Window *
+Ecore_Win32_Window *
 ecore_win32_window_new(Ecore_Win32_Window *parent,
                        int                 x,
                        int                 y,
@@ -147,7 +53,7 @@ ecore_win32_window_new(Ecore_Win32_Window *parent,
 }
 
 /* simulate X11 override windows */
-EAPI Ecore_Win32_Window *
+Ecore_Win32_Window *
 ecore_win32_window_override_new(Ecore_Win32_Window *parent,
                                 int                 x,
                                 int                 y,
@@ -160,51 +66,26 @@ ecore_win32_window_override_new(Ecore_Win32_Window *parent,
                                           WS_POPUP);
 }
 
-EAPI void
+void
 ecore_win32_window_del(Ecore_Win32_Window *window)
 {
-   Ecore_Win32_Window *w;
-
    if (!window) return;
 
-   ecore_list_first_goto(_ecore_win32_windows_list);
-   while ((w = ecore_list_next(_ecore_win32_windows_list)))
-     {
-       if (w == window)
-         {
-            ecore_list_remove(_ecore_win32_windows_list);
-            break;
-         }
-     }
-/*    ecore_list_remove(_ecore_win32_windows_list); */
-
-   switch (((struct _Ecore_Win32_Window *)window)->backend)
-     {
-     case ECORE_WIN32_BACKEND_DIRECTDRAW:
-       ecore_win32_ddraw_shutdown(window);
-       break;
-     case ECORE_WIN32_BACKEND_DIRECTDRAW_16:
-/*        ecore_win32_ddraw_shutdown(window); */
-       break;
-     case ECORE_WIN32_BACKEND_DIRECT3D:
-       printf ("d3d shut 0 \n");
-       ecore_win32_direct3d_shutdown(window);
-       printf ("d3d shut 1 \n");
-       break;
-     case ECORE_WIN32_BACKEND_GLEW:
-       ecore_win32_glew_shutdown(window);
-       break;
-     default:
-       break;
-     }
-
    DestroyWindow(((struct _Ecore_Win32_Window *)window)->window);
    free(window);
    printf ("ecore_win32_window_del\n");
 }
 
+void *
+ecore_win32_window_hwnd_get(Ecore_Win32_Window *window)
+{
+   if (!window) return NULL;
+
+   return ((struct _Ecore_Win32_Window *)window)->window;
+}
+
 /*
-EAPI void
+void
 ecore_win32_window_configure(Ecore_Win32_Window        *window,
                              Ecore_Win32_Window_Z_Order order,
                              int                        x,
@@ -235,7 +116,7 @@ ecore_win32_window_configure(Ecore_Win32_Window        *window,
 }
 */
 
-EAPI void
+void
 ecore_win32_window_move(Ecore_Win32_Window *window,
                         int                 x,
                         int                 y)
@@ -256,7 +137,7 @@ ecore_win32_window_move(Ecore_Win32_Window *window,
               TRUE);
 }
 
-EAPI void
+void
 ecore_win32_window_resize(Ecore_Win32_Window *window,
                           int                 width,
                           int                 height)
@@ -308,7 +189,7 @@ ecore_win32_window_resize(Ecore_Win32_Window *window,
    printf ("ecore_win32_window_resize 4 : %d %d\n", width, height);
 }
 
-EAPI void
+void
 ecore_win32_window_move_resize(Ecore_Win32_Window *window,
                                int                 x,
                                int                 y,
@@ -325,10 +206,10 @@ ecore_win32_window_move_resize(Ecore_Win32_Window *window,
    w = ((struct _Ecore_Win32_Window *)window);
    rect.left = 0;
    rect.top = 0;
-   if (width < w->min_width) width = w->min_width;
-   if (width > w->max_width) width = w->max_width;
-   if (height < w->min_height) height = w->min_height;
-   if (height > w->max_height) height = w->max_height;
+   if ((unsigned int)width < w->min_width) width = w->min_width;
+   if ((unsigned int)width > w->max_width) width = w->max_width;
+   if ((unsigned int)height < w->min_height) height = w->min_height;
+   if ((unsigned int)height > w->max_height) height = w->max_height;
    printf ("ecore_win32_window_move_resize 1 : %d %d\n", width, height);
    rect.right = width;
    rect.bottom = height;
@@ -342,7 +223,7 @@ ecore_win32_window_move_resize(Ecore_Win32_Window *window,
               TRUE);
 }
 
-EAPI void
+void
 ecore_win32_window_geometry_get(Ecore_Win32_Window *window,
                                 int                *x,
                                 int                *y,
@@ -395,7 +276,7 @@ ecore_win32_window_geometry_get(Ecore_Win32_Window *window,
    if (height) *height = h;
 }
 
-EAPI void
+void
 ecore_win32_window_size_get(Ecore_Win32_Window *window,
                             int                *width,
                             int                *height)
@@ -422,7 +303,7 @@ ecore_win32_window_size_get(Ecore_Win32_Window *window,
    if (height) *height = rect.bottom - rect.top;
 }
 
-EAPI void
+void
 ecore_win32_window_size_min_set(Ecore_Win32_Window *window,
                                 unsigned int        min_width,
                                 unsigned int        min_height)
@@ -437,7 +318,7 @@ ecore_win32_window_size_min_set(Ecore_Win32_Window *window,
    w->min_height = min_height;
 }
 
-EAPI void
+void
 ecore_win32_window_size_min_get(Ecore_Win32_Window *window,
                                 unsigned int       *min_width,
                                 unsigned int       *min_height)
@@ -446,13 +327,13 @@ ecore_win32_window_size_min_get(Ecore_Win32_Window *window,
 
    if (!window) return;
 
-   printf ("ecore_win32_window_size_min_get : %p  %d %d\n", window, w->min_width, w->min_height);
    w = (struct _Ecore_Win32_Window *)window;
+   printf ("ecore_win32_window_size_min_get : %p  %d %d\n", window, w->min_width, w->min_height);
    if (min_width) *min_width = w->min_width;
    if (min_height) *min_height = w->min_height;
 }
 
-EAPI void
+void
 ecore_win32_window_size_max_set(Ecore_Win32_Window *window,
                                 unsigned int        max_width,
                                 unsigned int        max_height)
@@ -467,7 +348,7 @@ ecore_win32_window_size_max_set(Ecore_Win32_Window *window,
    w->max_height = max_height;
 }
 
-EAPI void
+void
 ecore_win32_window_size_max_get(Ecore_Win32_Window *window,
                                 unsigned int       *max_width,
                                 unsigned int       *max_height)
@@ -476,13 +357,13 @@ ecore_win32_window_size_max_get(Ecore_Win32_Window *window,
 
    if (!window) return;
 
-   printf ("ecore_win32_window_size_max_get : %p  %d %d\n", window, w->max_width, w->max_height);
    w = (struct _Ecore_Win32_Window *)window;
+   printf ("ecore_win32_window_size_max_get : %p  %d %d\n", window, w->max_width, w->max_height);
    if (max_width) *max_width = w->max_width;
    if (max_height) *max_height = w->max_height;
 }
 
-EAPI void
+void
 ecore_win32_window_size_base_set(Ecore_Win32_Window *window,
                                  unsigned int        base_width,
                                  unsigned int        base_height)
@@ -497,7 +378,7 @@ ecore_win32_window_size_base_set(Ecore_Win32_Window *window,
    w->base_height = base_height;
 }
 
-EAPI void
+void
 ecore_win32_window_size_base_get(Ecore_Win32_Window *window,
                                  unsigned int       *base_width,
                                  unsigned int       *base_height)
@@ -506,13 +387,13 @@ ecore_win32_window_size_base_get(Ecore_Win32_Window *window,
 
    if (!window) return;
 
-   printf ("ecore_win32_window_size_base_get : %p  %d %d\n", window, w->base_width, w->base_height);
    w = (struct _Ecore_Win32_Window *)window;
+   printf ("ecore_win32_window_size_base_get : %p  %d %d\n", window, w->base_width, w->base_height);
    if (base_width) *base_width = w->base_width;
    if (base_height) *base_height = w->base_height;
 }
 
-EAPI void
+void
 ecore_win32_window_size_step_set(Ecore_Win32_Window *window,
                                  unsigned int        step_width,
                                  unsigned int        step_height)
@@ -527,7 +408,7 @@ ecore_win32_window_size_step_set(Ecore_Win32_Window *window,
    w->step_height = step_height;
 }
 
-EAPI void
+void
 ecore_win32_window_size_step_get(Ecore_Win32_Window *window,
                                  unsigned int       *step_width,
                                  unsigned int       *step_height)
@@ -536,15 +417,15 @@ ecore_win32_window_size_step_get(Ecore_Win32_Window *window,
 
    if (!window) return;
 
-   printf ("ecore_win32_window_size_step_get : %p  %d %d\n", window, w->step_width, w->step_height);
    w = (struct _Ecore_Win32_Window *)window;
+   printf ("ecore_win32_window_size_step_get : %p  %d %d\n", window, w->step_width, w->step_height);
    if (step_width) *step_width = w->step_width;
    if (step_height) *step_height = w->step_height;
 }
 
 /* TODO: ecore_win32_window_shaped_set */
 
-EAPI void
+void
 ecore_win32_window_show(Ecore_Win32_Window *window)
 {
    if (!window) return;
@@ -555,7 +436,7 @@ ecore_win32_window_show(Ecore_Win32_Window *window)
 }
 
 /* FIXME: seems to block the taskbar */
-EAPI void
+void
 ecore_win32_window_hide(Ecore_Win32_Window *window)
 {
    if (!window) return;
@@ -564,7 +445,7 @@ ecore_win32_window_hide(Ecore_Win32_Window *window)
    ShowWindow(((struct _Ecore_Win32_Window *)window)->window, SW_HIDE);
 }
 
-EAPI void
+void
 ecore_win32_window_raise(Ecore_Win32_Window *window)
 {
    if (!window) return;
@@ -575,7 +456,7 @@ ecore_win32_window_raise(Ecore_Win32_Window *window)
                 SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE);
 }
 
-EAPI void
+void
 ecore_win32_window_lower(Ecore_Win32_Window *window)
 {
    if (!window) return;
@@ -586,7 +467,7 @@ ecore_win32_window_lower(Ecore_Win32_Window *window)
                 SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE);
 }
 
-EAPI void
+void
 ecore_win32_window_title_set(Ecore_Win32_Window *window,
                              const char         *title)
 {
@@ -597,7 +478,7 @@ ecore_win32_window_title_set(Ecore_Win32_Window *window,
    SetWindowText(((struct _Ecore_Win32_Window *)window)->window, title);
 }
 
-EAPI void
+void
 ecore_win32_window_focus_set(Ecore_Win32_Window *window)
 {
    if (!window) return;
@@ -605,7 +486,7 @@ ecore_win32_window_focus_set(Ecore_Win32_Window *window)
    SetFocus(((struct _Ecore_Win32_Window *)window)->window);
 }
 
-EAPI void
+void
 ecore_win32_window_iconified_set(Ecore_Win32_Window *window,
                                  int                 on)
 {
@@ -631,7 +512,7 @@ ecore_win32_window_iconified_set(Ecore_Win32_Window *window,
    ew->iconified = on;
 }
 
-EAPI void
+void
 ecore_win32_window_borderless_set(Ecore_Win32_Window *window,
                                   int                 on)
 {
@@ -670,7 +551,7 @@ ecore_win32_window_borderless_set(Ecore_Win32_Window *window,
    ew->borderless = on;
 }
 
-EAPI void
+void
 ecore_win32_window_fullscreen_set(Ecore_Win32_Window *window,
                                   int                 on)
 {
@@ -714,7 +595,7 @@ ecore_win32_window_fullscreen_set(Ecore_Win32_Window *window,
      }
 }
 
-EAPI void
+void
 ecore_win32_window_cursor_set(Ecore_Win32_Window *window,
                               Ecore_Win32_Cursor *cursor)
 {
@@ -722,7 +603,7 @@ ecore_win32_window_cursor_set(Ecore_Win32_Window *window,
                 GCL_HCURSOR, (LONG)cursor);
 }
 
-EAPI void
+void
 ecore_win32_window_state_set(Ecore_Win32_Window       *window,
                              Ecore_Win32_Window_State *state,
                              unsigned int              num)
@@ -780,7 +661,7 @@ ecore_win32_window_state_set(Ecore_Win32_Window       *window,
      }
 }
 
-EAPI void
+void
 ecore_win32_window_state_request_send(Ecore_Win32_Window      *window,
                                       Ecore_Win32_Window_State state,
                                       unsigned int             set)
@@ -889,7 +770,7 @@ ecore_win32_window_state_request_send(Ecore_Win32_Window      *window,
      }
 }
 
-EAPI void
+void
 ecore_win32_window_type_set(Ecore_Win32_Window      *window,
                             Ecore_Win32_Window_Type  type)
 {
@@ -924,3 +805,118 @@ ecore_win32_window_type_set(Ecore_Win32_Window      *window,
        break;
      }
 }
+
+
+/***** Private functions definitions *****/
+
+static Ecore_Win32_Window *
+ecore_win32_window_internal_new(Ecore_Win32_Window *parent,
+                                int                 x,
+                                int                 y,
+                                int                 width,
+                                int                 height,
+                                DWORD               style)
+{
+   RECT                        rect;
+   struct _Ecore_Win32_Window *w;
+   int                         minimal_width;
+   int                         minimal_height;
+
+   w = (struct _Ecore_Win32_Window *)calloc(1, sizeof(struct _Ecore_Win32_Window));
+   if (!w)
+     return NULL;
+
+   printf (" *** ecore_win32_window_new : %p  %d %d %d\n",
+           w,
+           width, height, GetSystemMetrics(SM_CXMIN));
+   rect.left = 0;
+   rect.top = 0;
+   rect.right = width;
+   rect.bottom = height;
+   if (!AdjustWindowRect(&rect, style, FALSE))
+     {
+        free(w);
+        return NULL;
+     }
+   printf (" * ecore : new debut : %ld %d %d\n",
+           rect.right - rect.left, GetSystemMetrics(SM_CXMIN), GetSystemMetrics(SM_CYMIN));
+
+   minimal_width = GetSystemMetrics(SM_CXMIN);
+   minimal_height = GetSystemMetrics(SM_CYMIN);
+/*    if (((rect.right - rect.left) < minimal_width) || */
+/*        ((rect.bottom - rect.top) < minimal_height)) */
+/*      { */
+/*         fprintf (stderr, "[Ecore] [Win32] ERROR !!\n"); */
+/*         fprintf (stderr, "                Wrong size %ld\n", rect.right - rect.left); */
+/*         free(w); */
+/*         return NULL; */
+/*      } */
+   if ((rect.right - rect.left) < minimal_width)
+     {
+       rect.right = rect.left + minimal_width;
+     }
+
+   w->window = CreateWindow(ECORE_WIN32_WINDOW_CLASS, "",
+                            style,
+                            x, y,
+                            rect.right - rect.left,
+                            rect.bottom - rect.top,
+                            parent ? ((struct _Ecore_Win32_Window *)parent)->window : NULL,
+                            NULL, _ecore_win32_instance, NULL);
+   if (!w->window)
+     {
+        free(w);
+        return NULL;
+     }
+
+   SetLastError(0);
+   if (!SetWindowLong(w->window, GWL_USERDATA, (LONG)w) && (GetLastError() != 0))
+     {
+        DestroyWindow(w->window);
+        free(w);
+        return NULL;
+     }
+
+   w->min_width   = 0;
+   w->min_height  = 0;
+   w->max_width   = 32767;
+   w->max_height  = 32767;
+   w->base_width  = -1;
+   w->base_height = -1;
+   w->step_width  = -1;
+   w->step_height = -1;
+
+   w->state.iconified         = 0;
+   w->state.modal             = 0;
+   w->state.sticky            = 0;
+   w->state.maximized_vert    = 0;
+   w->state.maximized_horz    = 0;
+   w->state.shaded            = 0;
+   w->state.hidden            = 0;
+   w->state.fullscreen        = 0;
+   w->state.above             = 0;
+   w->state.below             = 0;
+   w->state.demands_attention = 0;
+
+   w->type.desktop = 0;
+   w->type.dock    = 0;
+   w->type.toolbar = 0;
+   w->type.menu    = 0;
+   w->type.utility = 0;
+   w->type.splash  = 0;
+   w->type.dialog  = 0;
+   w->type.normal  = 0;
+
+   w->pointer_is_in = 0;
+   w->borderless    = 0;
+   w->iconified     = 0;
+   w->fullscreen    = 0;
+
+   printf (" *** ecore_win32_window_new fin : (%d %d) (%d %d)\n",
+           w->min_width,
+           w->min_height,
+           w->max_width,
+           w->max_height);
+
+   return w;
+}
