@@ -298,8 +298,10 @@ struct _Ecore_X_Event_Key_Down
    char   *keysymbol;
    char   *key_compose;
    int     modifiers;
+   int	   same_screen;
    Ecore_X_Window  win;
    Ecore_X_Window  event_win;
+   Ecore_X_Window  root_win;
    Ecore_X_Time    time;
 };
 
@@ -309,8 +311,10 @@ struct _Ecore_X_Event_Key_Up
    char   *keysymbol;
    char   *key_compose;
    int     modifiers;
+   int	   same_screen;
    Ecore_X_Window  win;
    Ecore_X_Window  event_win;
+   Ecore_X_Window  root_win;
    Ecore_X_Time    time;
 };
 
@@ -319,11 +323,13 @@ struct _Ecore_X_Event_Mouse_Button_Down
    int     button;
    int     modifiers;
    int     x, y;
+   int	   same_screen;
    struct {
       int  x, y;
    } root;
    Ecore_X_Window  win;
    Ecore_X_Window  event_win;
+   Ecore_X_Window  root_win;
    Ecore_X_Time    time;
    unsigned int    double_click : 1;
    unsigned int    triple_click : 1;
@@ -334,11 +340,13 @@ struct _Ecore_X_Event_Mouse_Button_Up
    int     button;
    int     modifiers;
    int     x, y;
+   int	   same_screen;
    struct {
       int  x, y;
    } root;
    Ecore_X_Window  win;
    Ecore_X_Window  event_win;
+   Ecore_X_Window  root_win;
    Ecore_X_Time    time;
    unsigned int    double_click : 1;
    unsigned int    triple_click : 1;
@@ -348,11 +356,13 @@ struct _Ecore_X_Event_Mouse_Move
 {
    int     modifiers;
    int     x, y;
+   int	   same_screen;
    struct {
       int  x, y;
    } root;
    Ecore_X_Window  win;
    Ecore_X_Window  event_win;
+   Ecore_X_Window  root_win;
    Ecore_X_Time    time;
 };
 
@@ -360,13 +370,15 @@ struct _Ecore_X_Event_Mouse_In
 {
    int                  modifiers;
    int                  x, y;
+   int			same_screen;
    struct {
       int  x, y;
    } root;
    Ecore_X_Window               win;
    Ecore_X_Window               event_win;
-   Ecore_X_Event_Mode   mode;
-   Ecore_X_Event_Detail detail;
+   Ecore_X_Window		root_win;
+   Ecore_X_Event_Mode		mode;
+   Ecore_X_Event_Detail		detail;
    Ecore_X_Time                 time;
 };
 
@@ -374,13 +386,15 @@ struct _Ecore_X_Event_Mouse_Out
 {
    int                  modifiers;
    int                  x, y;
+   int			same_screen;
    struct {
       int  x, y;
    } root;
    Ecore_X_Window               win;
    Ecore_X_Window               event_win;
-   Ecore_X_Event_Mode   mode;
-   Ecore_X_Event_Detail detail;
+   Ecore_X_Window		root_win;
+   Ecore_X_Event_Mode		mode;
+   Ecore_X_Event_Detail		detail;
    Ecore_X_Time                 time;
 };
 
@@ -390,6 +404,7 @@ struct _Ecore_X_Event_Mouse_Wheel
    int z; /* ...,-2,-1 = down, 1,2,... = up */
    int modifiers;
    int x, y;
+   int same_screen;
 
    struct {
       int x, y;
@@ -397,6 +412,7 @@ struct _Ecore_X_Event_Mouse_Wheel
 
    Ecore_X_Window win;
    Ecore_X_Window event_win;
+   Ecore_X_Window root_win;
    Ecore_X_Time   time;
 };
 
@@ -446,18 +462,21 @@ struct _Ecore_X_Event_Window_Create
 struct _Ecore_X_Event_Window_Destroy
 {
    Ecore_X_Window  win;
+   Ecore_X_Window  event_win;
    Ecore_X_Time    time;
 };
 
 struct _Ecore_X_Event_Window_Hide
 {
    Ecore_X_Window  win;
+   Ecore_X_Window  event_win;
    Ecore_X_Time    time;
 };
 
 struct _Ecore_X_Event_Window_Show
 {
    Ecore_X_Window  win;
+   Ecore_X_Window  event_win;
    Ecore_X_Time    time;
 };
 
@@ -471,6 +490,7 @@ struct _Ecore_X_Event_Window_Show_Request
 struct _Ecore_X_Event_Window_Reparent
 {
    Ecore_X_Window  win;
+   Ecore_X_Window  event_win; 
    Ecore_X_Window  parent;
    Ecore_X_Time    time;
 };
@@ -478,6 +498,7 @@ struct _Ecore_X_Event_Window_Reparent
 struct _Ecore_X_Event_Window_Configure
 {
    Ecore_X_Window  win;
+   Ecore_X_Window  event_win;
    Ecore_X_Window  abovewin;
    int             x, y, w, h;
    int             border;
@@ -489,6 +510,7 @@ struct _Ecore_X_Event_Window_Configure
 struct _Ecore_X_Event_Window_Configure_Request
 {
    Ecore_X_Window  win;
+   Ecore_X_Window  parent_win;
    Ecore_X_Window  abovewin;
    int             x, y, w, h;
    int             border;
@@ -500,6 +522,7 @@ struct _Ecore_X_Event_Window_Configure_Request
 struct _Ecore_X_Event_Window_Gravity
 {
    Ecore_X_Window  win;
+   Ecore_X_Window  event_win;
    Ecore_X_Time    time;
 };
 
@@ -969,6 +992,20 @@ typedef enum _Ecore_X_Window_Configure_Mask {
    ECORE_X_WINDOW_CONFIGURE_MASK_STACK_MODE     = (1 << 6)
 } Ecore_X_Window_Configure_Mask;
 
+typedef enum _Ecore_X_Virtual_Keyboard_State {
+    ECORE_X_VIRTUAL_KEYBOARD_STATE_UNKNOWN = 0,
+    ECORE_X_VIRTUAL_KEYBOARD_STATE_OFF,
+    ECORE_X_VIRTUAL_KEYBOARD_STATE_ON,
+    ECORE_X_VIRTUAL_KEYBOARD_STATE_ALPHA,
+    ECORE_X_VIRTUAL_KEYBOARD_STATE_NUMERIC,
+    ECORE_X_VIRTUAL_KEYBOARD_STATE_PIN,
+    ECORE_X_VIRTUAL_KEYBOARD_STATE_PHONE_NUMBER,
+    ECORE_X_VIRTUAL_KEYBOARD_STATE_HEX,
+    ECORE_X_VIRTUAL_KEYBOARD_STATE_TERMINAL,
+    ECORE_X_VIRTUAL_KEYBOARD_STATE_PASSWORD
+} Ecore_X_Virtual_Keyboard_State;
+
+
 /* Window layer constants */
 #define ECORE_X_WINDOW_LAYER_BELOW 2
 #define ECORE_X_WINDOW_LAYER_NORMAL 4
@@ -992,6 +1029,7 @@ EAPI void             ecore_x_flush(void);
 EAPI void             ecore_x_sync(void);
 EAPI void             ecore_x_killall(Ecore_X_Window root);
 EAPI void             ecore_x_kill(Ecore_X_Window win);
+EAPI int              ecore_x_dpi_get(void);
 
 EAPI Ecore_X_Time     ecore_x_current_time_get(void);
 
@@ -1048,6 +1086,8 @@ EAPI int              ecore_x_dnd_begin(Ecore_X_Window source, unsigned char *da
 EAPI int              ecore_x_dnd_drop(void);
 EAPI void             ecore_x_dnd_send_status(int will_accept, int suppress, Ecore_X_Rectangle rectangle, Ecore_X_Atom action);
 EAPI void             ecore_x_dnd_send_finished(void);
+EAPI void             ecore_x_dnd_source_action_set(Ecore_X_Atom action);
+EAPI Ecore_X_Atom     ecore_x_dnd_source_action_get(void);
 
 EAPI Ecore_X_Window   ecore_x_window_new(Ecore_X_Window parent, int x, int y, int w, int h);
 EAPI Ecore_X_Window   ecore_x_window_override_new(Ecore_X_Window parent, int x, int y, int w, int h);
@@ -1258,7 +1298,8 @@ EAPI void            ecore_x_window_client_sniff(Ecore_X_Window win);
 EAPI void            ecore_x_atom_get_prefetch(const char *name);
 EAPI void            ecore_x_atom_get_fetch(void);
 EAPI Ecore_X_Atom    ecore_x_atom_get(const char *name);
-
+EAPI void            ecore_x_atoms_get(const char **names, int num, Ecore_X_Atom *atoms);
+       
 
 
 EAPI void            ecore_x_icccm_init(void);
@@ -1389,6 +1430,7 @@ EAPI void                ecore_x_netwm_showing_desktop_set(Ecore_X_Window root,
 EAPI void                ecore_x_netwm_client_list_set(Ecore_X_Window root, Ecore_X_Window *p_clients, unsigned int n_clients);
 EAPI void                ecore_x_netwm_client_list_stacking_set(Ecore_X_Window root, Ecore_X_Window *p_clients, unsigned int n_clients);
 EAPI void                ecore_x_netwm_client_active_set(Ecore_X_Window root, Ecore_X_Window win);
+EAPI void                ecore_x_netwm_client_active_request(Ecore_X_Window root, Ecore_X_Window win, int type, Ecore_X_Window current_win);
 EAPI void                ecore_x_netwm_name_set(Ecore_X_Window win, const char *name);
 EAPI void                ecore_x_netwm_name_get_prefetch(Ecore_X_Window window);
 EAPI void                ecore_x_netwm_name_get_fetch(void);
@@ -1448,6 +1490,7 @@ EAPI void                ecore_x_netwm_window_type_set(Ecore_X_Window win, Ecore
 EAPI void                ecore_x_netwm_window_type_get_prefetch(Ecore_X_Window window);
 EAPI void                ecore_x_netwm_window_type_get_fetch(void);
 EAPI int                 ecore_x_netwm_window_type_get(Ecore_X_Window win, Ecore_X_Window_Type *type);
+EAPI int                 ecore_x_netwm_window_types_get(Ecore_X_Window win, Ecore_X_Window_Type **types);
 EAPI int                 ecore_x_netwm_allowed_action_isset(Ecore_X_Window win, Ecore_X_Action action);
 EAPI void                ecore_x_netwm_allowed_action_set(Ecore_X_Window win, Ecore_X_Action *action, unsigned int num);
 EAPI void                ecore_x_netwm_allowed_action_get_prefetch(Ecore_X_Window window);
@@ -1474,7 +1517,12 @@ EAPI void                ecore_x_netwm_desktop_request_send(Ecore_X_Window win,
 
 EAPI void                ecore_x_e_init(void);
 EAPI void                ecore_x_e_frame_size_set(Ecore_X_Window win, int fl, int fr, int ft, int fb);
-
+EAPI void                ecore_x_e_virtual_keyboard_set(Ecore_X_Window win, int is_keyboard);
+EAPI int                 ecore_x_e_virtual_keyboard_get(Ecore_X_Window win);
+EAPI void                ecore_x_e_virtual_keyboard_state_set(Ecore_X_Window win, Ecore_X_Virtual_Keyboard_State state);
+EAPI Ecore_X_Virtual_Keyboard_State ecore_x_e_virtual_keyboard_state_get(Ecore_X_Window win);
+EAPI void                ecore_x_e_virtual_keyboard_state_send(Ecore_X_Window win, Ecore_X_Virtual_Keyboard_State state);
+       
 
 EAPI void                ecore_x_xinerama_query_screens_prefetch(void);
 EAPI void                ecore_x_xinerama_query_screens_fetch(void);
@@ -1537,6 +1585,10 @@ EAPI int  ecore_x_pointer_control_set(int accel_num, int accel_denom, int thresh
 EAPI void ecore_x_pointer_control_get_prefetch(void);
 EAPI void ecore_x_pointer_control_get_fetch(void);
 EAPI int  ecore_x_pointer_control_get(int *accel_num, int *accel_denom, int *threshold);
+EAPI int  ecore_x_pointer_mapping_set(unsigned char *map, int nmap);
+EAPI void ecore_x_pointer_mapping_get_prefetch(void);
+EAPI void ecore_x_pointer_mapping_get_fetch(void);
+EAPI int  ecore_x_pointer_mapping_get(unsigned char *map, int nmap);
 EAPI int  ecore_x_pointer_grab(Ecore_X_Window win);
 EAPI int  ecore_x_pointer_confine_grab(Ecore_X_Window win);
 EAPI void ecore_x_pointer_ungrab(void);
@@ -1680,7 +1732,11 @@ EAPI void              ecore_x_dpms_timeout_standby_set(unsigned int new_timeout
 EAPI void              ecore_x_dpms_timeout_suspend_set(unsigned int new_timeout);
 EAPI void              ecore_x_dpms_timeout_off_set(unsigned int new_timeout);
 
-
+EAPI int ecore_x_test_fake_key_down(const char *key);
+EAPI int ecore_x_test_fake_key_up(const char *key);
+EAPI int ecore_x_test_fake_key_press(const char *key);
+EAPI const char *ecore_x_keysym_string_get(int keysym);
+   
 #ifdef __cplusplus
 }
 #endif
